/*  gtkDialog 1.04
 *  Copyright (C) 1998 Antti Huotari <ahuotari@cc.hut.fi>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 */

#include <string.h> 
#include <gtk/gtk.h>

#define NBUTTONS (sizeof button_names / sizeof (struct button_name))

static gint button_number = 0;

struct button_name
{
    gchar *button_name;
    gchar *xpm_button_filename;
}   button_names[] =
    {
        "PREVIOUS", "/usr/share/gtkdialog/xpm/previous.xpm",
        "NEXT", "/usr/share/gtkdialog/xpm/next.xpm",
        "LEFT", "/usr/share/gtkdialog/xpm/left.xpm",
        "RIGHT", "/usr/share/gtkdialog/xpm/right.xpm",
        "CENTER", "/usr/share/gtkdialog/xpm/center.xpm",
        "UP", "/usr/share/gtkdialog/xpm/up.xpm",
        "DOWN", "/usr/share/gtkdialog/xpm/down.xpm",
        "CANCEL", "/usr/share/gtkdialog/xpm/cancel.xpm",
        "TERMINATE", "/usr/share/gtkdialog/xpm/terminate.xpm",
        "SUSPEND", "/usr/share/gtkdialog/xpm/suspend.xpm",
        "REDO", "/usr/share/gtkdialog/xpm/redo.xpm",
        "UNDO", "/usr/share/gtkdialog/xpm/undo.xpm",
        "RETRY", "/usr/share/gtkdialog/xpm/redo.xpm",
        "ABORT", "/usr/share/gtkdialog/xpm/abort.xpm",
        "EXIT", "/usr/share/gtkdialog/xpm/exit.xpm",
        "DENY", "/usr/share/gtkdialog/xpm/deny.xpm",
        "IGNORE", "/usr/share/gtkdialog/xpm/ignore.xpm",
        "DISABLE", "/usr/share/gtkdialog/xpm/disable.xpm",
        "ENABLE", "/usr/share/gtkdialog/xpm/enable.xpm",
        "OK", "/usr/share/gtkdialog/xpm/ok.xpm",
        "YES", "/usr/share/gtkdialog/xpm/yes.xpm",
        "NO", "/usr/share/gtkdialog/xpm/no.xpm",
        "APPLY", "/usr/share/gtkdialog/xpm/apply.xpm"
    };

gint 
close_application (GtkWidget *widget, gpointer data)
{
    button_number = 0;
    gtk_main_quit ();
}

void 
clicked_button (GtkWidget *widget, gpointer data)
{
    button_number = (gint) data;
    gtk_main_quit ();      
}

gint
timeout_return (gpointer data)
{
    button_number = (gint) data;
    gtk_main_quit ();
}

/*
 * xpm_label_box -function is from the GTK Tutorial by 
 * Ian Main <imain@gtk.org> and Tony Gale <gale@gtk.org>
 * The GTK Tutorial is Copyright (C) 1997 Ian Main.
 * Copyright (C) 1998 Tony Gale. 
 */
GtkWidget*
xpm_label_box (GtkWidget *parent, gchar *xpm_filename, gchar *label_text)
{
    GtkWidget *box1;
    GtkWidget *label;
    GtkWidget *pixmapwid;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GtkStyle *style;
    
    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_border_width (GTK_CONTAINER (box1), 2);

    style = gtk_widget_get_style (parent);

    pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
                                         &style->bg[GTK_STATE_NORMAL],
                                         xpm_filename);

    pixmapwid = gtk_pixmap_new (pixmap, mask);

    label = gtk_label_new (label_text);

    gtk_box_pack_start (GTK_BOX (box1),
                        pixmapwid, FALSE, FALSE, 3);

    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);

    gtk_widget_show(pixmapwid);
    gtk_widget_show(label);

    return box1;
}

GtkWidget*
parse_and_make_buttons (gchar *p[], GtkWidget *parent)
{
    GtkWidget *box;
    GtkWidget *box2;
    GtkWidget *button;
    static gchar *parsed_button;
    gint array_length, button_text_length;
    gint i = 0, n = 0, FOUND = FALSE;

    box = gtk_hbox_new (TRUE, 5);

    array_length = strlen (p[0]);
    parsed_button = g_malloc (1 + array_length);

    do
    {
        sscanf (*p, "%s", &parsed_button);

        for (n = 0; n < NBUTTONS; n++)
        {
            if (!g_strcasecmp(button_names[n].button_name, 
                              (gpointer) &parsed_button))
            {
                FOUND = TRUE;
                break;
            }
            FOUND = FALSE;         
        }

        if (FOUND)
            box2 = xpm_label_box (parent, button_names[n].xpm_button_filename, 
                                  (gpointer) &parsed_button);
        else
            box2 = xpm_label_box (parent, "/usr/share/gtkdialog/xpm/green2.xpm", 
                                  (gpointer) &parsed_button);

	gtk_widget_show (box2);
        
        button = gtk_button_new ();
        
        gtk_container_add (GTK_CONTAINER (button), box2);
        gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0); 
        gtk_widget_show (button);
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                            GTK_SIGNAL_FUNC (clicked_button), 
                            (gpointer) button_number + 1);

        button_text_length = strlen ((gpointer) &parsed_button);
        i += button_text_length + 1;
        *p += button_text_length + 1;

        button_number++;
    }   while (i < array_length);  

    return box;
}

GtkWidget*
insert_message (gchar *p, GtkWidget *window)
{
    GtkWidget *vbox;
    GtkWidget *hbox;    
    GtkWidget *text;
    GtkWidget *table;
    GtkWidget *vscrollbar;
    
    gtk_window_set_policy (GTK_WINDOW(window), FALSE, TRUE, FALSE);
    
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (hbox), 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show (hbox);

    table = gtk_table_new (2, 2, FALSE);
    gtk_table_set_row_spacing (GTK_TABLE (table), 0, 2);
    gtk_table_set_col_spacing (GTK_TABLE (table), 0, 2);
    gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
    gtk_widget_show (table);

    text = gtk_text_new (NULL, NULL);
    gtk_text_set_editable (GTK_TEXT (text), FALSE);
    gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (text);

    vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
    gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
                      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (vscrollbar);

    gtk_text_freeze (GTK_TEXT (text));
    gtk_widget_realize (text);
   
    gtk_text_insert (GTK_TEXT (text), NULL, NULL,
                     NULL, (gpointer) p, -1);

    gtk_text_set_word_wrap(GTK_TEXT (text), TRUE);

    return vbox;
}

void
usage ()
{
    g_print ("usage: gtkdialog [options]\n");
    g_print ("options: \n\t");
    g_print ("-b <buttons>\n\t\t");
    g_print ("This option tells the program which buttons to use.\n\t\t");
    g_print ("For example: gtkdialog -b \"YES NO\"\n\t");
    g_print ("-t <title>\n\t\t");
    g_print ("This option gives a title for the window.\n\t\t");
    g_print ("For example: gtkdialog -t \"Popup message\"\n\t");
    g_print ("-m <message>\n\t\t");
    g_print ("This option gives a message for the program to display.\n\t\t");
    g_print ("For example: gtkdialog -m \"Do you want to quit?\"\n\t");
    g_print ("-timeout <interval in seconds>\n\t\t");
    g_print ("This option tells the program to return a value one greater\n\t\t");
    g_print ("than the amount of buttons after the interval has elapsed.\n\t\t");
    g_print ("For example: gtkdialog -timeout 60\n\t");
    g_print ("-V\n\t\t");
    g_print ("Display the version number of the program.\n\n\t");
    g_print ("If you don't give a -b or -t or -m option, the default values are:\n\t");
    g_print ("-b \"OK\" -t \"gtkDialog\" -m \"Blah blah blah ...\"\n\n\t"); 
    g_print ("You probably want to use these options bunched together:\n\t");
    g_print ("gtkdialog -b \"YES NO\" -t \"Popup message\" -m \"Do you want to quit?\"\n"); 
}

int 
main (int argc, char *argv[])
{
    GtkWidget *window;
    GtkWidget *box1;
    GtkWidget *box2;    
    GtkWidget *separator;
    gchar *button_arg, *title_arg, *message_arg, *version_number;
    guint32 timeout_arg;
    gint i;

    version_number = g_malloc (1 + strlen ("1.04"));
    strcpy (version_number, "1.04");

    button_arg = g_malloc (1 + strlen ("OK"));
    strcpy (button_arg, "OK");
    title_arg = g_malloc (1 + strlen ("gtkDialog"));
    strcpy (title_arg, "gtkDialog");
    message_arg = g_malloc (1 + strlen ("Blah blah blaaah ..."));
    strcpy (message_arg, "Blah blah blaaah ...");

    timeout_arg = 0;

    
    if (argc == 2 && !strcmp(argv[1], "-V"))
    {
        g_print ("gtkDialog Version %s\n", version_number);
        exit (0);
    }

    if (argc == 2 || argc == 4 || argc == 6 || argc == 8 || argc > 9)
    {
        usage ();
        exit (0);
    }

    for (i = 1; i < argc; i = i + 2) 
    {
        if (!strcmp (argv[i], "-b"))
        {
            g_free (button_arg);
            button_arg = g_malloc (1 + strlen (argv[i + 1]));
            strcpy (button_arg, argv[i + 1]);            
        }
        else if (!strcmp (argv[i], "-t"))
        {
            g_free (title_arg);
            title_arg = g_malloc (1 + strlen (argv[i + 1]));
            strcpy (title_arg, argv[i + 1]);
        }
        else if (!strcmp (argv[i], "-m"))
        {
            g_free (message_arg);
            message_arg = g_malloc (1 + strlen (argv[i + 1]));
            strcpy (message_arg, argv[i + 1]);
        }
        else if (!strcmp (argv[i], "-timeout"))
            timeout_arg = 1000 * atoi (argv[i + 1]);
    }
   
    gtk_init (&argc, &argv);

    gtk_rc_parse ("/usr/share/gtkdialog/grc");    
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), title_arg);

    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
                        GTK_SIGNAL_FUNC (close_application), NULL);

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
                        GTK_SIGNAL_FUNC (close_application), NULL);  

    gtk_container_border_width (GTK_CONTAINER (window), 10);
    gtk_widget_realize (window);

    box1 = insert_message (message_arg, window); 

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 10);    
    gtk_widget_show (separator);

    box2 = parse_and_make_buttons (&button_arg, window);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
    
    gtk_widget_show (box2);
    gtk_widget_show (window);
    if (timeout_arg != 0)
        gtk_timeout_add (timeout_arg, timeout_return, 
                         (gpointer) button_number + 1);
 
    gtk_main ();
  
    return button_number;
}

